%
% simple matlab script illustrating ravine search-based scatterometer
% wind retrieval resulting in multiple ambiguities.  See Chapter 16 of 
% (F. Ulaby and D.G. Long, Microwave Radar and Radiometric Remote 
% Sensing, ISBN: 978-0-472-11935-6, University of Michigan Press, 
% Ann Arbor, Michigan, 2013.)
% 
% The code computes the C-band backscatter as a function of wind 
% speed and direction for each fan-beam antenna at a fixed 
% across-track distance where the antenna azimuth angles are 45, 90, and
% 135 deg relative to the scatterometer ground track.  For a
% particular wind speed and two directions (0 and 45 deg in 
% meteorological convention, the sigma-0 is computed and simulated
% Monte Carlo noise is added with Kp =0.05.  The LLS(U, phi) objective
% function is minimized with the error between the true wind vector
% and the wind alias closest to the true wind vector determined
%
% written by D.G. Long at BYU Dec 2011
%

% set true wind
true_spd=10;   % m/s
true_dir=30;   % deg

% set measurement geometry
% note that incidence angle of all beams is set equal for simplicity
azimuth=[45,90,135]';  % beam azimuth angles
incidence=[30,30,30]'; % beam incidence angles

% generate true noise-free sigma-0 values
z=zeros(size(azimuth));
arg=[true_spd+z,true_dir+z,azimuth,incidence];
sigs=CMOD5(arg)';

% set noise level
Kp=0.0; % noise free

% noise to sigma-0 measurements
sigs=sigs.*(1+Kp*rand(size(sigs)));

% prepare argument to ravine search
rarg=[azimuth,incidence,sigs];

% find ambiguities using ravine search
[ambs, plt]=ravine_search_CMOD5(rarg);

disp(sprintf('True speed %6.2f and direction %6.1f',true_spd,true_dir));
disp(sprintf('N=%d ambiguities',size(ambs,1)))
disp('    Spd     Dir     LS');
for k=1:size(ambs,1)
  disp(sprintf(' %d %6.2f %6.1f %f',k,ambs(k,:)));
end

figure(2); clf;
subplot(2,1,1)
plot(plt(:,2),plt(:,3))
title(sprintf('Ravine LS versus direction: %d ambiguities',size(ambs,1)))
xlabel('wind direction (deg)');
ylabel('LS objective function value');
hold on;
% note: * show locations of minimum LS rror -- these define the ambiguities
plot(ambs(:,2),ambs(:,3),'r*');
hold off

subplot(2,1,2)
plot(plt(:,2),plt(:,1))
title('Ravine speed versus direction')
xlabel('wind direction (deg)');
ylabel('estimated wine speed (deg)');
hold on;
% note: direction associated with ambiguity is at location * on curve
plot(ambs(:,2),ambs(:,1),'r*');
hold off

% generate plot of speed versus direction given sigma^0 for each beam

% set up directional optimization for plottting speed versus direction 
% given sigma0
global search_geom
options=optimset('TolX',0.1); % 0.1 m/s speed precision

figure(1); clf
dirs1=0:2:360;
spds1=zeros(size(dirs1));

for beam=1:size(arg)
  col='b';
  if beam==2
    col='r';
  end
  if beam==3
    col='g';
  end  

  for idir=1:length(dirs1)
    % set global variable to pass function arguments
    search_geom=[dirs1(idir),azimuth(beam),incidence(beam),sigs(beam)];
    % call matlab optimization of speed at a particular direction
    [spds1(idir) objs1]=fminbnd(@dirmin,1,50,options);
  end
  
  plot(dirs1,spds1,col);
  hold on;
end
% mark true wind
plot(true_dir,true_spd,'k+');
% mark ambiguities
plot(ambs(:,2),ambs(:,1),'ko');
hold off;
title(sprintf('Backscatter for beam azimuths: blue=%d red=%d green=%d deg',azimuth));
xlabel('Wind direction (deg)');
ylabel('Wind speed (m/s)');

% save graphics
figure(1)
print -dpng ravine_1.png
figure(2)
print -dpng ravine_2.png
